from netgen.geom2d import *
from netgen.meshing import MeshingParameters
from ngsolve import *
from ngsolve.internal import visoptions

from lakemesh import MakeLakeMesh



true_compile = True

h = 0.4

t = 0
tend = 10
dt = 0.0002

g = 10
ref_h = 0.1

order=1

mesh = MakeLakeMesh("lake.pts", "lake.edges", hmax=h)

# geo = SplineGeometry()
# L=5
# pnts = [ (-L,-L), (L,-L), (L,L), (-L,L) ]
# pnums = [geo.AppendPoint(*p) for p in pnts]
# bottom = geo.Append ( ["line", pnums[0], pnums[1]], bc="bottomtop")
# right = geo.Append ( ["line", pnums[1], pnums[2]], bc="leftright")
# top = geo.Append ( ["line", pnums[3], pnums[2]], leftdomain=0, rightdomain=1, copy=bottom, bc="bottomtop")
# left = geo.Append ( ["line", pnums[0], pnums[3]], leftdomain=0, rightdomain=1, copy=right, bc="leftright")

# ngmesh = geo.GenerateMesh(maxh=h)
# mesh = Mesh(ngmesh)

n=specialcf.normal(mesh.dim)

fes = L2(mesh,order=order,dim=1+mesh.dim)

gfU = GridFunction(fes)

U0 = CoefficientFunction((ref_h*(1+0.5*exp(-2*((x-1)**2+(y-5)**2))),0,0))
gfU.Set(U0)

cf_h, cf_hvx, cf_hvy = gfU

cf_vx = cf_hvx/cf_h
cf_vy = cf_hvy/cf_h

Draw(cf_h-ref_h,mesh,"h",sd=int(order/2))
Draw(CoefficientFunction((cf_vx,cf_vy)),mesh,"v",sd=int(order/2))

U,V = fes.TnT()
h, hvx, hvy = U
h_t, hvx_t, hvy_t = V

Ubnd = CoefficientFunction((h,0,0))

Uhat = U.Other(bnd=Ubnd)

def pos(v):
    return IfPos(v,v,0)

def f(U):
    h, hvx, hvy = U
    h = pos(h)
    vx = hvx/h
    vy = hvy/h
    return CoefficientFunction(((hvx,hvy),
                                (hvx*vx + 0.5*g*h, hvx*vy),
                                (hvx*vy, hvy*vy + 0.5*g*h)),dims=(3,2))

def fmax(a,b):
    return 1

def fhatn(U):
    return (0.5*f(U)+0.5*f(Uhat))*n + fmax(U,Uhat)/2*(U-Uhat)

a = BilinearForm(fes, nonassemble=True)
a += SymbolicBFI (- InnerProduct(f(U),grad(V).trans).Compile(true_compile,wait=True))
a += SymbolicBFI (InnerProduct(fhatn(U),V).Compile(true_compile,wait=True), element_boundary=True)

w = gfU.vec.CreateVector()

input("")
cnt = 0 
with TaskManager():
    while t < tend:
        a.Apply (gfU.vec, w)
        fes.SolveM (rho=CoefficientFunction((1,1,1,1)), vec=w)
        gfU.vec.data -= dt * w
        t += dt
        if cnt % 100 == 0:
            Redraw(blocking=True)
        print("\rt : ",t,end="")
        cnt +=1
        # input("")
    
